package appointment.backing;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.TreeSet;

import org.apache.commons.logging.LogFactory;

public class AppointmentBook implements Serializable{

	private static final long serialVersionUID = -448928342262059127L;
	private Set providers = new TreeSet();
	TreeSet  daysAppointments = new TreeSet();
	private Map bookPages = new TreeMap();
    private Date currentDate = Util.getRoundDate(new Date());
    Calendar cal = Calendar.getInstance();
    private List selectedProviders = new ArrayList();
   
    public void rollDay( boolean forward) {
    	cal.setTime(currentDate);
    	cal.roll(Calendar.DATE, forward);
    	setCurrentDate(cal.getTime());
    }
    
    public void rollWeek( boolean forward) {
    	cal.setTime(currentDate);
    	for (int i = 0; i < 7; ++i)
    		cal.roll(Calendar.DATE, forward);
    	setCurrentDate(cal.getTime());
    	
    }
    
    public void rollMonth( boolean forward) {
    	
     	cal.setTime(currentDate);
    	cal.roll(Calendar.MONTH, forward);
    	setCurrentDate(cal.getTime());
    }
    
    
	public Map getBookPages() {
		return bookPages;
	}

	public void setBookPages(Map bookColumns) {
		this.bookPages = bookColumns;
	}

	public Date getCurrentDate() {
		return currentDate;
	}

	public void setCurrentDate(Date currentDate) {
		this.currentDate = Util.getRoundDate(currentDate);
	}

	public void setProviders(Set providers) {
		this.providers = providers;
	}

	public AppointmentBook() {
		
	}
	
	/** 
	 * @param provider
	 */
	public void addProvider(Provider provider) {
		
		providers.add(provider);
		
	}
	
	public List getProviders() {
		return new ArrayList(providers);
	}
	

	public void addProviderAppointments( ProviderAppointments appts) {
		Provider p = appts.getProvider();
		Date d = appts.getDate();
		if ( bookPages .get(d) == null) 
			bookPages.put(d, new TreeMap() );
		SortedMap map = (SortedMap) bookPages.get(d);
		map.put(p, appts);
	}
	
	public Map getCurrentAppointmentMap() {
		SortedMap map = (SortedMap) bookPages.get(getCurrentDate());
	 	if (map == null) {
	 		map = new TreeMap();
	 		bookPages.put(getCurrentDate(), map);
	 		LogFactory.getLog(getClass()).info("created new map "+map+"for "+getCurrentDate() );
	 	} else {
	 		LogFactory.getLog(getClass()).info("using existing map "+map+" for "+getCurrentDate() );
		 		
	 	}
	 	
	 	return map;
	}
	
	public Collection  getOneRowCurrentAppointments() {
		List l = new ArrayList();
		l.add(getCurrentAppointments() );
		return l;
	}
	
	public Collection getCurrentAppointments() {
		Map map = getCurrentAppointmentMap();
	 	List l = new ArrayList();
		for (Iterator iter = getSelectedProviders().iterator(); iter.hasNext();) {
			Provider prov = (Provider) iter.next();
			if ( map.get(prov) == null ) {
				ProviderAppointments pa  = new ProviderAppointments();
				pa.setProvider(prov);
				l.add(pa);
				map.put(prov,pa);
			} else {
				l.add(map.get(prov));
			}
				
		}
		return l;
	}

	public void setSelectedProviders(List l) {
		// TODO Auto-generated method stub
		selectedProviders = l;
	}

	public List getSelectedProviders() {
		return selectedProviders;
	}
	
}
