package appointment.backing;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.logging.Logger;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ProviderAppointments implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3110463614010197532L;
	private Provider provider;
	private Period period;
	private int  interval = 15;
	Calendar cal = Calendar.getInstance();
	private Date date ;
	List slots = null;
	
	private SortedSet appts = new TreeSet();
	private Log log = LogFactory.getLog(this.getClass());
	
	public ProviderAppointments() {
		
		date = Util.getRoundDate(new Date());
		period = new Period(new Time(8,0), new Time(19,0)  );
	}
	
	public int getInterval() {
		return interval;
	}

	public void setInterval(int interval) {
		if (interval > 0)
			this.interval = interval;
	}

	

	public SortedSet getAppts() {
		return appts;
	}

	public void setAppts(SortedSet appts) {
		this.appts = appts;
	}

	public Period getPeriod() {
		return period;
	}

	public void setPeriod(Period period) {
		this.period = period;
	}

	public  List getAppointmentSlots() {
		if (slots != null)
			return slots;
		slots = new ArrayList();
		Time curr  = getPeriod().getStart();
		Time last = getPeriod().getEnd();
		Iterator apptIter = appts.iterator();
		Appointment appt = null, displayedApp = null;
		if (apptIter.hasNext()) {
			appt = (Appointment) apptIter.next();
		}
		log .info("period.start ="+curr + " period.end "+last);
		
		while ( curr.before(last)) {
			AppointmentSlot slot = null;
			Time currEnd = curr.getNextTime(getInterval());
			log.info("start =" + curr + " currEnd = " + currEnd);
			if (appt!=null && appt.getPeriod().overlaps(curr, currEnd) 
					&& displayedApp != appt ) {
				slot = createFilledAppointmentSlot(appt,  curr, currEnd );
				displayedApp = appt;
				if (apptIter.hasNext()) {
					appt = (Appointment) apptIter.next();
				}
				
			} else   {
				slot = createEmptyAppointmentSlot( curr, currEnd );
			}
			slots.add(slot);
			curr = currEnd;
		}
		return slots;
	
	}

	
	private AppointmentSlot createEmptyAppointmentSlot(Time start, Time end) {
		// TODO Auto-generated method stub
		AppointmentSlot slot = new AppointmentSlot( new Appointment(), start, end);
		return slot;
	}

	private AppointmentSlot createFilledAppointmentSlot(Appointment ap, Time start,Time end) {
		// TODO Auto-generated method stub
		AppointmentSlot slot = new AppointmentSlot(ap, start, end);
		
		return slot;
	}

	public Provider getProvider() {
		return provider;
	}

	public void setProvider(Provider provider) {
		this.provider = provider;
	}
	
	public void addAppointment(Appointment ap) {
		appts.add(ap);
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		 
		this.date = Util.getRoundDate(date);
	}
}
